#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-


################################################
#############   MACHINE LEARNING   #############
##############     -  2016 -    ################
################################################

"""
Contact: Adrian Ulges (adrian.ulges@hs-rm.de)
"""

import numpy as np
import misc


"""
  Expectation Maximization
  -------------------------
  an interface for Expectation Maximization clustering.
  No commandline arguments -> call like this:

  > python EM.py
"""



class ExpectationMaximization:
    """
    a class for Expectation Maximization (EM). Given an n x m data matrix X (with lines
    representing feature vectors), compute the means and variances of clusters.
    The number of clusters K is fixed.

    - means is a (K,m)-Array containing the K cluster centers
    - vars  is a (K,m)-Array containing the K cluster's variances in each dimension
            (NOTE: We assume diagonal covariance matrices, i.e. all correlations between
                   variables are zero, and only the variances matter).
    - NOTE: The priors P(c) of each cluster is simply 1/K

    The attribute 'self.model' should pack together means and variances:

    > self.model = (means,vars)

    Run clustering using train().
    """

    def __init__(self, K, iterations=100):
        """
        class constructor.

        @type K: int
        @param K: the number of clusters. Must be > 0.
        @type iterations: int
        @param K: the number of iterations to run. Must be > 0.
        """
        self.K = K
        self.iterations = iterations
        self.model = None


    def train(self, X):
        """
        runs an EM clustering on a given set of samples / features.

        @type X: np.array (n x m)
        @param X: each row in X corresponds to one feature vector.
                  There should be more feature vectors than clusters
                  (i.e., n > self.K)
        """
        raise NotImplementedError()

        # FIXME: compute means and vars using EM

        self.model = means,vars







        
if __name__ == "__main__":

    X = misc.generate_test_data()

    misc.plot(X, model=None)

    em = ExpectationMaximization(K=3)

    em.train(X)

    misc.plot(X, model=em.model)
